//------------------------------------------------------------------------------
//
//  dp_main_tbc.v : displaport main-link data generate module
//
//  LICENSE : "as-is"
//  TakeshiNagashima(T.NG) caramelgete@gmail.com
//------------------------------------------------------------------------------
//  2009/dec/15 release 0.0  
//  2010/jan/28 release 0.1  display timinig generate test
//
//------------------------------------------------------------------------------

module dp_main_tbc(
	input	[7:0]	TX_RED,			// in    [CRTC] red[7:0]
	input	[7:0]	TX_GRN,			// in    [CRTC] grn[7:0]
	input	[7:0]	TX_BLU,			// in    [CRTC] blu[7:0]
	input			TX_HS,			// in    [CRTC] hsync
	input			TX_VS,			// in    [CRTC] vsync
	input			TX_DE,			// in    [CRTC] de/#blank
	input			TX_CLK,			// in    [CRTC] pixel clock
//	output			TX_SYNC,		// out   [CRTC] pixel clock

	input	[23:0]	dp_mvid,		// in    [TBC] mvid[23:0]
	input	[23:0]	dp_nvid,		// in    [TBC] nvid[23:0]
	input	[7:0]	dp_maud,		// in    [TBC] maud[7:0]
	input	[15:0]	dp_htotal,		// in    [TBC] htotal[15:0]
	input	[15:0]	dp_vtotal,		// in    [TBC] vtotal[15:0]
	input	[15:0]	dp_hstart,		// in    [TBC] hstart[15:0]
	input	[15:0]	dp_vstart,		// in    [TBC] vstart[15:0]
	input	[15:0]	dp_hwidth,		// in    [TBC] hwidth[15:0]
	input	[15:0]	dp_vheight,		// in    [TBC] vheight[15:0]
	input	[15:0]	dp_hsync,		// in    [TBC] hsync[15:0] ({hs_pol,hs_wid[14:0]})
	input	[15:0]	dp_vsync,		// in    [TBC] vsync[15:0] ({vs_pol,vs_wid[14:0]})
	input	[7:0]	dp_misc0,		// in    [TBC] misc0[7:0]
	input	[7:0]	dp_misc1,		// in    [TBC] misc1[7:0]

	output	[8:0]	tbc_data0,		// out   [TBC] data0[8:0]
	output	[8:0]	tbc_data1,		// out   [TBC] data1[8:0]
	output	[8:0]	tbc_data2,		// out   [TBC] data2[8:0]
	output	[8:0]	tbc_data3,		// out   [TBC] data3[8:0]

	input			CLK,			// in    [MAIN] system clock
	input			SYSRST_N		// in    [MAIN] #reset
);


//--------------------------------------------------------------
//  override parameter

	parameter	dp_vbid_vs  =8'b00_0_1_0_0_0_1;	// !de,sync (wo HDCP)
	parameter	dp_vbid_hs  =8'b00_0_1_0_0_0_0;	// de,sync  (wo HDCP)
	parameter	dp_vbid_idle=8'b00_0_1_0_0_0_0;	// blank    (wo HDCP)

	parameter	K237=9'b1_111_10111;
	parameter	K277=9'b1_111_11011;
	parameter	K280=9'b1_000_11100;
	parameter	K281=9'b1_001_11100;
	parameter	K282=9'b1_010_11100;
	parameter	K283=9'b1_011_11100;
	parameter	K284=9'b1_100_11100;
	parameter	K285=9'b1_101_11100;
	parameter	K286=9'b1_110_11100;
	parameter	K287=9'b1_111_11100;
	parameter	K297=9'b1_111_11101;
	parameter	K307=9'b1_111_11110;
	parameter	D102=9'b0_010_01010;
	parameter	D116=9'b0_110_01011;

	localparam	code_bs  =K285;	// defalut symbol blank start
	localparam	code_be  =K277;	// defalut symbol blank end
	localparam	code_ss  =K282;	// defalut symbol secondary start
	localparam	code_se  =K297;	// defalut symbol secondary end
	localparam	code_fs  =K307;	// defalut symbol fill start
	localparam	code_fe  =K237;	// defalut symbol fill end
	localparam	code_sr  =K280;	// defalut symbol scramble reset
	localparam	code_cpbs=K281;	// defalut symbol content protection bs
	localparam	code_cpsr=K283;	// defalut symbol content protection sr

	localparam	code_ebs =K285;	// enhanced symbol blank start
	localparam	code_ebe =K277;	// enhanced symbol blank end
	localparam	code_ess =K282;	// enhanced symbol secondary start
	localparam	code_ese =K297;	// enhanced symbol secondary end
	localparam	code_efs =K307;	// enhanced symbol fill start
	localparam	code_efe =K237;	// enhanced symbol fill end
	localparam	code_esr =K280;	// enhanced symbol scramble reset
	localparam	code_ecp =K281;	// enhanced symbol content protection
	localparam	code_ebf =K283;	// enhanced symbol 

//--------------------------------------------------------------
//  local parameter

//--------------------------------------------------------------
//  signal

//--------------------------------------------------------------
//  design

	// ---- re-timing ----

	wire	disp_dclk_sync;
	wire	[23:0] disp_dclk_w;
	wire	[3:0] disp_dclk_sync_w;
	wire	[23:0] disp_reg_dclk_w;
	reg		[23:0] disp_dclk_r;
	reg		[3:0] disp_dclk_sync_r;
	reg		[23:0] disp_reg_dclk_r;

	wire	disp_hsbp;
	wire	disp_hsav;
	wire	disp_hde1;
	wire	disp_heav;
	wire	disp_hend;
	wire	disp_vsbp;
	wire	disp_vsav;
	wire	disp_veav;
	wire	disp_vend;


//	assign TX_SYNC=disp_dclk_sync_r[0];
	assign disp_dclk_sync=disp_dclk_sync_r[3];

	always @(posedge CLK or negedge SYSRST_N)
	begin
		if (SYSRST_N==1'b0)
			begin
				disp_dclk_r[23:0] <= 24'b0;
				disp_dclk_sync_r[3:0] <= 4'b0;
				disp_reg_dclk_r[23:0] <= 24'b0;
			end
		else
			begin
				disp_dclk_r[23:0] <= disp_dclk_w[23:0];
				disp_dclk_sync_r[3:0] <= disp_dclk_sync_w[3:0];
				disp_reg_dclk_r[23:0] <= disp_reg_dclk_w[23:0];
			end
	end

	wire	[24:0] disp_dclk_add;

	assign disp_dclk_add[24:0]={1'b0,disp_dclk_r[23:0]}+{1'b0,disp_reg_dclk_r[23:0]};

	assign disp_dclk_w[23:0]={9'b0,disp_dclk_add[14:0]};
	assign disp_dclk_sync_w[3]=(disp_dclk_add[15]==2'b0) ? 1'b0 : 1'b1;
	assign disp_dclk_sync_w[2:0]=(disp_dclk_add[15]==2'b0) ? {1'b0,disp_dclk_sync_r[2:1]} : 3'b111;

	assign disp_reg_dclk_w[23:0]=dp_mvid[23:0];

	// ---- ----

	wire	[7:0] tx_red_in;
	wire	[7:0] tx_grn_in;
	wire	[7:0] tx_blu_in;
	wire	tx_hs_in;
	wire	tx_vs_in;
	wire	tx_de_in;
	wire	tx_req_in;

	wire	[7:0] tx_red_in_w;
	wire	[7:0] tx_grn_in_w;
	wire	[7:0] tx_blu_in_w;
	wire	tx_hs_in_w;
	wire	tx_vs_in_w;
	wire	tx_de_in_w;
	wire	tx_req_in_w;

	reg		[7:0] tx_red_in_r;
	reg		[7:0] tx_grn_in_r;
	reg		[7:0] tx_blu_in_r;
	reg		tx_hs_in_r;
	reg		tx_vs_in_r;
	reg		tx_de_in_r;
	reg		tx_req_in_r;

	wire	[108:0] tbc_wr_data;
	wire	[7:0] tbc_wr_count;
	wire	tbc_wr;

	wire	[23:0] tx_data00_in_w;
	wire	[23:0] tx_data01_in_w;
	wire	[23:0] tx_data02_in_w;
	wire	[23:0] tx_data03_in_w;
	wire	[11:0] tx_data_count_w;
	wire	[8:0] tx_data_wr_w;
	reg		[23:0] tx_data00_in_r;
	reg		[23:0] tx_data01_in_r;
	reg		[23:0] tx_data02_in_r;
	reg		[23:0] tx_data03_in_r;
	reg		[11:0] tx_data_count_r;
	reg		[8:0] tx_data_wr_r;

	wire	[7:0] tx_hs_in0_w;
	wire	[7:0] tx_hs_in1_w;
	wire	[7:0] tx_vs_in0_w;
	wire	[7:0] tx_vs_in1_w;
	wire	[7:0] tx_de_in0_w;
	wire	[7:0] tx_de_in1_w;
	wire	tx_hs_w;
	wire	tx_vs_w;
	wire	tx_de_w;
	wire	tx_hs_req_w;
	wire	tx_vs_req_w;
	wire	tx_de_req_w;
	wire	[1:0] tx_ss_in_w;
	wire	tx_ss_req_w;
	reg		[7:0] tx_hs_in0_r;
	reg		[7:0] tx_hs_in1_r;
	reg		[7:0] tx_vs_in0_r;
	reg		[7:0] tx_vs_in1_r;
	reg		[7:0] tx_de_in0_r;
	reg		[7:0] tx_de_in1_r;
	reg		tx_hs_r;
	reg		tx_vs_r;
	reg		tx_de_r;
	reg		tx_hs_req_r;
	reg		tx_vs_req_r;
	reg		tx_de_req_r;
	reg		[1:0] tx_ss_in_r;
	reg		tx_ss_req_r;

	assign tbc_wr_data[107:0]={12'b0,tx_data03_in_r[23:0],tx_data02_in_r[23:0],tx_data01_in_r[23:0],tx_data00_in_r[23:0]};
	assign tbc_wr_count[7:0]=tx_data_wr_r[7:0];
	assign tbc_wr=tx_data_wr_r[8];

	always @(posedge CLK or negedge SYSRST_N)
	begin
		if (SYSRST_N==1'b0)
			begin
				tx_red_in_r[7:0] <= 8'b0;
				tx_grn_in_r[7:0] <= 8'b0;
				tx_blu_in_r[7:0] <= 8'b0;
				tx_hs_in_r <= 1'b0;
				tx_vs_in_r <= 1'b0;
				tx_de_in_r <= 1'b0;
				tx_req_in_r <= 1'b0;

				tx_data00_in_r[23:0] <= 24'b0;
				tx_data01_in_r[23:0] <= 24'b0;
				tx_data02_in_r[23:0] <= 24'b0;
				tx_data03_in_r[23:0] <= 24'b0;
				tx_data_count_r[11:0] <= 12'b0;
				tx_data_wr_r[8:0] <= 9'b0;

				tx_hs_in0_r[7:0] <= 8'b0;
				tx_hs_in1_r[7:0] <= 8'b0;
				tx_vs_in0_r[7:0] <= 8'b0;
				tx_vs_in1_r[7:0] <= 8'b0;
				tx_de_in0_r[7:0] <= 8'b0;
				tx_de_in1_r[7:0] <= 8'b0;
				tx_hs_r <= 1'b0;
				tx_vs_r <= 1'b0;
				tx_de_r <= 1'b0;
				tx_hs_req_r <= 1'b0;
				tx_vs_req_r <= 1'b0;
				tx_de_req_r <= 1'b0;
				tx_ss_in_r[1:0] <= 2'b0;
				tx_ss_req_r <= 1'b0;
			end
		else
			begin
				tx_red_in_r[7:0] <= tx_red_in_w[7:0];
				tx_grn_in_r[7:0] <= tx_grn_in_w[7:0];
				tx_blu_in_r[7:0] <= tx_blu_in_w[7:0];
				tx_hs_in_r <= tx_hs_in_w;
				tx_vs_in_r <= tx_vs_in_w;
				tx_de_in_r <= tx_de_in_w;
				tx_req_in_r <= tx_req_in_w;

				tx_data00_in_r[23:0] <= tx_data00_in_w[23:0];
				tx_data01_in_r[23:0] <= tx_data01_in_w[23:0];
				tx_data02_in_r[23:0] <= tx_data02_in_w[23:0];
				tx_data03_in_r[23:0] <= tx_data03_in_w[23:0];
				tx_data_count_r[11:0] <= tx_data_count_w[11:0];
				tx_data_wr_r[8:0] <= tx_data_wr_w[8:0];

				tx_hs_in0_r[7:0] <= tx_hs_in0_w[7:0];
				tx_hs_in1_r[7:0] <= tx_hs_in1_w[7:0];
				tx_vs_in0_r[7:0] <= tx_vs_in0_w[7:0];
				tx_vs_in1_r[7:0] <= tx_vs_in1_w[7:0];
				tx_de_in0_r[7:0] <= tx_de_in0_w[7:0];
				tx_de_in1_r[7:0] <= tx_de_in1_w[7:0];
				tx_hs_r <= tx_hs_w;
				tx_vs_r <= tx_vs_w;
				tx_de_r <= tx_de_w;
				tx_hs_req_r <= tx_hs_req_w;
				tx_vs_req_r <= tx_vs_req_w;
				tx_de_req_r <= tx_de_req_w;
				tx_ss_in_r[1:0] <= tx_ss_in_w[1:0];
				tx_ss_req_r <= tx_ss_req_w;
			end
	end

	assign tx_red_in_w[7:0]=(TX_CLK==1'b1) ? TX_RED[7:0] : tx_red_in_r[7:0];
	assign tx_grn_in_w[7:0]=(TX_CLK==1'b1) ? TX_GRN[7:0] : tx_grn_in_r[7:0];
	assign tx_blu_in_w[7:0]=(TX_CLK==1'b1) ? TX_BLU[7:0] : tx_blu_in_r[7:0];
	assign tx_hs_in_w=(TX_CLK==1'b1) ? TX_HS : tx_hs_in_r;
	assign tx_vs_in_w=(TX_CLK==1'b1) ? TX_VS : tx_vs_in_r;
	assign tx_de_in_w=(TX_CLK==1'b1) ? TX_DE : tx_de_in_r;
	assign tx_req_in_w=TX_CLK;

	assign tx_data00_in_w[23:0]=(TX_CLK==1'b1) & (tx_data_count_r[1:0]==2'b00) ? {tx_blu_in_r[7:0],tx_grn_in_r[7:0],tx_red_in_r[7:0]} : tx_data00_in_r[23:0];
	assign tx_data01_in_w[23:0]=(TX_CLK==1'b1) & (tx_data_count_r[1:0]==2'b01) ? {tx_blu_in_r[7:0],tx_grn_in_r[7:0],tx_red_in_r[7:0]} : tx_data01_in_r[23:0];
	assign tx_data02_in_w[23:0]=(TX_CLK==1'b1) & (tx_data_count_r[1:0]==2'b10) ? {tx_blu_in_r[7:0],tx_grn_in_r[7:0],tx_red_in_r[7:0]} : tx_data02_in_r[23:0];
	assign tx_data03_in_w[23:0]=(TX_CLK==1'b1) & (tx_data_count_r[1:0]==2'b11) ? {tx_blu_in_r[7:0],tx_grn_in_r[7:0],tx_red_in_r[7:0]} : tx_data03_in_r[23:0];
	assign tx_data_count_w[11:0]=
			(TX_CLK==1'b1) & (tx_de_in_r==1'b0) ? 12'b0 :
			(TX_CLK==1'b1) & (tx_de_in_r==1'b1) ? tx_data_count_r[11:0]+12'h01 :
			(TX_CLK==1'b0) ? tx_data_count_r[11:0] :
			12'b0;
	assign tx_data_wr_w[8]=(TX_CLK==1'b1) & (tx_de_in_r==1'b1) & (tx_data_count_r[1:0]==2'b11) ? 1'b1 : 1'b0;
	assign tx_data_wr_w[7:0]=(TX_CLK==1'b1) & (tx_de_in_r==1'b1) & (tx_data_count_r[1:0]==2'b11) ? tx_data_count_r[9:2] : tx_data_wr_r[7:0];

	assign tx_hs_in0_w[7]=
			(TX_CLK==1'b1) & (tx_hs_in0_r[3:0]==4'hf) ? 1'b1 :
			(TX_CLK==1'b1) & (tx_hs_in0_r[3:0]!=4'hf) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_hs_in0_r[7] :
			1'b0;
	assign tx_hs_in0_w[6:0]=
			(TX_CLK==1'b1) & (tx_hs_in0_r[3:0]==4'h0) & ({TX_HS,tx_hs_in_r}==2'b10) ? 7'h01 :
			(TX_CLK==1'b1) & (tx_hs_in0_r[3:0]==4'h0) & ({TX_HS,tx_hs_in_r}!=2'b10) ? 7'h0 :
			(TX_CLK==1'b1) & (tx_hs_in0_r[3:0]!=7'h0) ? tx_hs_in0_r[6:0]+7'h01 :
			(TX_CLK==1'b0) ? tx_hs_in0_r[6:0] :
			7'h0;
	assign tx_hs_in1_w[7]=
			(TX_CLK==1'b1) & (tx_hs_in1_r[3:0]==4'hf) ? 1'b1 :
			(TX_CLK==1'b1) & (tx_hs_in1_r[3:0]!=4'hf) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_hs_in1_r[7] :
			1'b0;
	assign tx_hs_in1_w[6:0]=
			(TX_CLK==1'b1) & (tx_hs_in1_r[3:0]==4'h0) & ({TX_HS,tx_hs_in_r}==2'b01) ? 7'h01 :
			(TX_CLK==1'b1) & (tx_hs_in1_r[3:0]==4'h0) & ({TX_HS,tx_hs_in_r}!=2'b01) ? 7'h0 :
			(TX_CLK==1'b1) & (tx_hs_in1_r[3:0]!=7'h0) ? tx_hs_in1_r[6:0]+7'h01 :
			(TX_CLK==1'b0) ? tx_hs_in1_r[6:0] :
			7'h0;
	assign tx_vs_in0_w[7]=
			(TX_CLK==1'b1) & (tx_vs_in0_r[3:0]==4'hf) ? 1'b1 :
			(TX_CLK==1'b1) & (tx_vs_in0_r[3:0]!=4'hf) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_vs_in0_r[7] :
			1'b0;
	assign tx_vs_in0_w[6:0]=
			(TX_CLK==1'b1) & (tx_vs_in0_r[3:0]==4'h0) & ({TX_VS,tx_vs_in_r}==2'b10) ? 7'h01 :
			(TX_CLK==1'b1) & (tx_vs_in0_r[3:0]==4'h0) & ({TX_VS,tx_vs_in_r}!=2'b10) ? 7'h0 :
			(TX_CLK==1'b1) & (tx_vs_in0_r[3:0]!=7'h0) ? tx_vs_in0_r[6:0]+7'h01 :
			(TX_CLK==1'b0) ? tx_vs_in0_r[6:0] :
			7'h0;
	assign tx_vs_in1_w[7]=
			(TX_CLK==1'b1) & (tx_vs_in1_r[3:0]==4'hf) ? 1'b1 :
			(TX_CLK==1'b1) & (tx_vs_in1_r[3:0]!=4'hf) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_vs_in1_r[7] :
			1'b0;
	assign tx_vs_in1_w[6:0]=
			(TX_CLK==1'b1) & (tx_vs_in1_r[3:0]==4'h0) & ({TX_VS,tx_vs_in_r}==2'b01) ? 7'h01 :
			(TX_CLK==1'b1) & (tx_vs_in1_r[3:0]==4'h0) & ({TX_VS,tx_vs_in_r}!=2'b01) ? 7'h0 :
			(TX_CLK==1'b1) & (tx_vs_in1_r[3:0]!=7'h0) ? tx_vs_in1_r[6:0]+7'h01 :
			(TX_CLK==1'b0) ? tx_vs_in1_r[6:0] :
			7'h0;
	assign tx_de_in0_w[7]=
			(TX_CLK==1'b1) & (tx_de_in0_r[3:0]==4'hf) ? 1'b1 :
			(TX_CLK==1'b1) & (tx_de_in0_r[3:0]!=4'hf) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_de_in0_r[7] :
			1'b0;
	assign tx_de_in0_w[6:0]=
			(TX_CLK==1'b1) & (tx_de_in0_r[3:0]==4'h0) & ({TX_DE,tx_de_in_r}==2'b10) ? 7'h01 :
			(TX_CLK==1'b1) & (tx_de_in0_r[3:0]==4'h0) & ({TX_DE,tx_de_in_r}!=2'b10) ? 7'h0 :
			(TX_CLK==1'b1) & (tx_de_in0_r[3:0]!=7'h0) ? tx_de_in0_r[6:0]+7'h01 :
			(TX_CLK==1'b0) ? tx_de_in0_r[6:0] :
			7'h0;
	assign tx_de_in1_w[7]=
			(TX_CLK==1'b1) & (tx_de_in1_r[3:0]==4'hf) ? 1'b1 :
			(TX_CLK==1'b1) & (tx_de_in1_r[3:0]!=4'hf) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_de_in1_r[7] :
			1'b0;
	assign tx_de_in1_w[6:0]=
			(TX_CLK==1'b1) & (tx_de_in1_r[3:0]==4'h0) & ({TX_DE,tx_de_in_r}==2'b01) ? 7'h01 :
			(TX_CLK==1'b1) & (tx_de_in1_r[3:0]==4'h0) & ({TX_DE,tx_de_in_r}!=2'b01) ? 7'h0 :
			(TX_CLK==1'b1) & (tx_de_in1_r[3:0]!=7'h0) ? tx_de_in1_r[6:0]+7'h01 :
			(TX_CLK==1'b0) ? tx_de_in1_r[6:0] :
			7'h0;
	assign tx_hs_w=
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b00) ? tx_hs_r :
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b01) ? 1'b1 :	// active low
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b10) ? 1'b0 :	// active low
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b11) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_hs_r :
			1'b0;
	assign tx_vs_w=
			(TX_CLK==1'b1) & ({tx_vs_in0_r[7],tx_vs_in1_r[7]}==2'b00) ? tx_vs_r :
			(TX_CLK==1'b1) & ({tx_vs_in0_r[7],tx_vs_in1_r[7]}==2'b01) ? 1'b1 :	// active low
			(TX_CLK==1'b1) & ({tx_vs_in0_r[7],tx_vs_in1_r[7]}==2'b10) ? 1'b0 :	// active low
			(TX_CLK==1'b1) & ({tx_vs_in0_r[7],tx_vs_in1_r[7]}==2'b11) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_vs_r :
			1'b0;
	assign tx_de_w=
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b00) ? tx_de_r :
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b10) ? 1'b1 :	// active high
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b01) ? 1'b0 :	// active high
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b11) ? 1'b0 :
			(TX_CLK==1'b0) ? tx_de_r :
			1'b0;

	assign tx_hs_req_w=
		//	(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b01) ? 1'b1 :	// active high
		//	(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}!=2'b01) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b01) ? 1'b0 :	// idle
		//	(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}!=2'b01) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}!=2'b01) ? tx_hs_req_r :	// line end
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b01) ? 1'b0 :
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}!=2'b01) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b10) ? 1'b1 :	// start
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}!=2'b01) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}!=2'b10) ? 1'b0 :	// 
			(TX_CLK==1'b0) ? 1'b0 : // tx_hs_req_r :
			1'b0;
	assign tx_vs_req_w=
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b00) ? 1'b0 :
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b01) & (tx_ss_in_r[0]==1'b1) ? 1'b0 :	// blank
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b01) & (tx_ss_in_r[0]==1'b0) ? 1'b1 :	// de
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b10) ? 1'b0 :
			(TX_CLK==1'b1) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b11) ? 1'b0 :
			(TX_CLK==1'b0) ? 1'b0 :	// tx_vs_req_r :
			1'b0;
	assign tx_de_req_w=
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b00) ? 1'b0 :
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b10) ? 1'b1 :	// active high
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b01) ? 1'b0 :	// active high
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b11) ? 1'b0 :
			(TX_CLK==1'b0) ? 1'b0 :	// tx_de_req_r :
			1'b0;

	assign tx_ss_in_w[0]=
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b10) ? 1'b1 :	// de start
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}!=2'b10) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b10) ? 1'b0 :	// hs end
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}!=2'b10) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}!=2'b10) ? tx_ss_in_r[0] :	// else
			(TX_CLK==1'b0) ? tx_ss_in_r[0] :
			1'h0;
	assign tx_ss_in_w[1]=
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}==2'b10) ? 1'b1 :	// de start
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}!=2'b10) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b10) ? tx_ss_in_r[0] :	// hs end
			(TX_CLK==1'b1) & ({tx_de_in0_r[7],tx_de_in1_r[7]}!=2'b10) & ({tx_hs_in0_r[7],tx_hs_in1_r[7]}!=2'b10) ? tx_ss_in_r[1] :	// else
			(TX_CLK==1'b0) ? tx_ss_in_r[1] :
			1'h0;

	assign tx_ss_req_w=
			(TX_CLK==1'b1) &  (({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b10) & (tx_ss_in_r[1:0]==2'b10)) ? 1'b1 :	// request ss
			(TX_CLK==1'b1) & !(({tx_hs_in0_r[7],tx_hs_in1_r[7]}==2'b10) & (tx_ss_in_r[1:0]==2'b10)) ? 1'b0 :
			(TX_CLK==1'b0) ? 1'b0 :	// tx_ss_req_r :
			1'b0;

	// ---- ----

	wire	[7:0] tbc_rd_count;
	wire	[107:0] tbc_rd_data;

alt_ram_dp108x256 ram00_dp108x256 (
	.clock(CLK),
	.data(tbc_wr_data[107:0]),
	.rdaddress(tbc_rd_count[7:0]),
	.wraddress(tbc_wr_count[7:0]),
	.wren(tbc_wr),
	.q(tbc_rd_data[107:0])
);

	// ---- ----


	// ---- ----

	wire	[3:0] tbc_state_w;
	wire	[11:0] tbc_line_w;
	wire	tbc_line_end_w;
	wire	[3:0] tbc_data_sel_w;
	wire	[7:0] tbc_count_w;
	wire	tbc_count32_w;
	wire	[7:0] tbc_rd_count_w;
	wire	tbc_rd_end_w;
	wire	tbc_blank_end_w;
	wire	[1:0] tbc_de_count_w;
	wire	[107:0] tbc_rd_data00_w;
	wire	[107:0] tbc_rd_data01_w;
	wire	[35:0] tbc_disp00_w;
	wire	[35:0] tbc_disp01_w;
	wire	[35:0] tbc_disp02_w;
	wire	[35:0] tbc_disp03_w;

	wire	tbc_data_req_w;
	wire	[287:0] tbc_data00_w;
	wire	[287:0] tbc_data01_w;
	wire	[287:0] tbc_data02_w;
	wire	[287:0] tbc_data03_w;
	wire	[287:0] tbc_data00_out_w;
	wire	[287:0] tbc_data01_out_w;
	wire	[287:0] tbc_data02_out_w;
	wire	[287:0] tbc_data03_out_w;

	reg		[3:0] tbc_state_r;
	reg		[11:0] tbc_line_r;
	reg		tbc_line_end_r;
	reg		[3:0] tbc_data_sel_r;
	reg		[7:0] tbc_count_r;
	reg		tbc_count32_r;
	reg		[7:0] tbc_rd_count_r;
	reg		tbc_rd_end_r;
	reg		tbc_blank_end_r;
	reg		[1:0] tbc_de_count_r;
	reg		[107:0] tbc_rd_data00_r;
	reg		[107:0] tbc_rd_data01_r;
	reg		[35:0] tbc_disp00_r;
	reg		[35:0] tbc_disp01_r;
	reg		[35:0] tbc_disp02_r;
	reg		[35:0] tbc_disp03_r;

	reg		tbc_data_req_r;
	reg		[287:0] tbc_data00_r;
	reg		[287:0] tbc_data01_r;
	reg		[287:0] tbc_data02_r;
	reg		[287:0] tbc_data03_r;
	reg		[287:0] tbc_data00_out_r;
	reg		[287:0] tbc_data01_out_r;
	reg		[287:0] tbc_data02_out_r;
	reg		[287:0] tbc_data03_out_r;

	wire	[287:0] tbc_disp_de;
	wire	[287:0] tbc_disp;
	wire	[287:0] tbc_disp_hs;
	wire	[287:0] tbc_disp_vs;

	wire	[287:0] tbc_disp_sel0;
	wire	[287:0] tbc_disp_sel1;
	wire	[287:0] tbc_disp_sel2;
	wire	[287:0] tbc_disp_sel3;

	wire	[287:0] tbc_sync_vs;
	wire	[287:0] tbc_sync_ss0;
	wire	[287:0] tbc_sync_ss1;
	wire	[287:0] tbc_sync_ss2;
	wire	[287:0] tbc_sync_ss3;

	assign tbc_data0[8:0]=tbc_data00_out_r[9:0];
	assign tbc_data1[8:0]=tbc_data01_out_r[9:0];
	assign tbc_data2[8:0]=tbc_data02_out_r[9:0];
	assign tbc_data3[8:0]=tbc_data03_out_r[9:0];

	assign tbc_rd_count[7:0]=tbc_rd_count_r[7:0];

	always @(posedge CLK or negedge SYSRST_N)
	begin
		if (SYSRST_N==1'b0)
			begin
				tbc_state_r[3:0] <= 4'b0;
				tbc_line_r[11:0] <= 12'b0;
				tbc_line_end_r <= 1'b0;
				tbc_data_sel_r[3:0] <= 4'b0;
				tbc_count_r[7:0] <= 8'b0;
				tbc_count32_r <= 1'b0;
				tbc_rd_count_r[7:0] <= 8'b0;
				tbc_rd_end_r <= 1'b0;
				tbc_blank_end_r <= 1'b0;
				tbc_de_count_r[1:0] <= 8'b0;
				tbc_rd_data00_r[107:0] <= 108'b0;
				tbc_rd_data01_r[107:0] <= 108'b0;
				tbc_disp00_r[35:0] <= 36'b0;
				tbc_disp01_r[35:0] <= 36'b0;
				tbc_disp02_r[35:0] <= 36'b0;
				tbc_disp03_r[35:0] <= 36'b0;

				tbc_data_req_r <= 1'b0;
				tbc_data00_r[287:0] <= 288'b0;
				tbc_data01_r[287:0] <= 288'b0;
				tbc_data02_r[287:0] <= 288'b0;
				tbc_data03_r[287:0] <= 288'b0;
				tbc_data00_out_r[287:0] <= 288'b0;
				tbc_data01_out_r[287:0] <= 288'b0;
				tbc_data02_out_r[287:0] <= 288'b0;
				tbc_data03_out_r[287:0] <= 288'b0;
			end
		else
			begin
				tbc_state_r[3:0] <= tbc_state_w[3:0];
				tbc_line_r[11:0] <= tbc_line_w[11:0];
				tbc_line_end_r <= tbc_line_end_w;
				tbc_data_sel_r[3:0] <= tbc_data_sel_w[3:0];
				tbc_count_r[7:0] <= tbc_count_w[7:0];
				tbc_count32_r <= tbc_count32_w;
				tbc_rd_count_r[7:0] <= tbc_rd_count_w[7:0];
				tbc_rd_end_r <= tbc_rd_end_w;
				tbc_blank_end_r <= tbc_blank_end_w;
				tbc_de_count_r[1:0] <= tbc_de_count_w[1:0];
				tbc_rd_data00_r[107:0] <= tbc_rd_data00_w[107:0];
				tbc_rd_data01_r[107:0] <= tbc_rd_data01_w[107:0];
				tbc_disp00_r[35:0] <= tbc_disp00_w[35:0];
				tbc_disp01_r[35:0] <= tbc_disp01_w[35:0];
				tbc_disp02_r[35:0] <= tbc_disp02_w[35:0];
				tbc_disp03_r[35:0] <= tbc_disp03_w[35:0];

				tbc_data_req_r <= tbc_data_req_w;
				tbc_data00_r[287:0] <= tbc_data00_w[287:0];
				tbc_data01_r[287:0] <= tbc_data01_w[287:0];
				tbc_data02_r[287:0] <= tbc_data02_w[287:0];
				tbc_data03_r[287:0] <= tbc_data03_w[287:0];
				tbc_data00_out_r[287:0] <= tbc_data00_out_w[287:0];
				tbc_data01_out_r[287:0] <= tbc_data01_out_w[287:0];
				tbc_data02_out_r[287:0] <= tbc_data02_out_w[287:0];
				tbc_data03_out_r[287:0] <= tbc_data03_out_w[287:0];
			end
	end

	assign tbc_state_w[3:0]=
			(tbc_state_r[3:0]==4'b0000) & (tx_de_req_r==1'b1) ? 4'b1000 :
			(tbc_state_r[3:0]==4'b0000) & (tx_de_req_r==1'b0) & (tx_vs_req_r==1'b1) ? 4'b0001 :
			(tbc_state_r[3:0]==4'b0000) & (tx_de_req_r==1'b0) & (tx_vs_req_r==1'b0) &  ((tbc_line_end_r==1'b1) & (tx_hs_req_r==1'b1)) ? 4'b0010 :
			(tbc_state_r[3:0]==4'b0000) & (tx_de_req_r==1'b0) & (tx_vs_req_r==1'b0) & !((tbc_line_end_r==1'b1) & (tx_hs_req_r==1'b1)) ? 4'b0000 :
			(tbc_state_r[3:0]==4'b0001) & (tbc_count32_r==1'b0) ? 4'b0001 :
			(tbc_state_r[3:0]==4'b0001) & (tbc_count32_r==1'b1) ? 4'b0000 :
			(tbc_state_r[3:0]==4'b0010) ? 4'b0011 :
			(tbc_state_r[3:0]==4'b0011) & (tbc_count32_r==1'b0) ? 4'b0011 :
			(tbc_state_r[3:0]==4'b0011) & (tbc_count32_r==1'b1) ? 4'b0000 :
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]==2'b11) ? 4'b1001 :
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]!=2'b11) ? 4'b1000 :
			(tbc_state_r[3:0]==4'b1001) & (tbc_count_r[1:0]==2'b11) ? 4'b1010 :
			(tbc_state_r[3:0]==4'b1001) & (tbc_count_r[1:0]!=2'b11) ? 4'b1001 :
			(tbc_state_r[3:0]==4'b1010) & (tbc_count_r[2:0]==3'b111) ? 4'b1011 :
			(tbc_state_r[3:0]==4'b1010) & (tbc_count_r[2:0]!=3'b111) ? 4'b1010 :
			(tbc_state_r[3:0]==4'b1011) & (tbc_count_r[2:0]==3'b111) ? 4'b1100 :
			(tbc_state_r[3:0]==4'b1011) & (tbc_count_r[2:0]!=3'b111) ? 4'b1011 :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]==3'b111) & (tbc_rd_end_r==1'b1) ? 4'b0000 :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]==3'b111) & (tbc_rd_end_r==1'b0) ? 4'b1000 :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]!=3'b111) ? 4'b1100 :
			(tbc_state_r[3:0]==4'b1101) ? 4'b0000 :
			(tbc_state_r[3:0]==4'b1110) ? 4'b0000 :
			(tbc_state_r[3:0]==4'b1111) ? 4'b0000 :
			4'b0000;

	assign tbc_line_w[11:0]=
		//	(tx_ss_req_r==1'b1) ? 12'b0 :
		//	(tx_ss_req_r==1'b0) & (tx_de_req_r==1'b1) ? tbc_line_r[11:0]+12'h01 :
		//	(tx_ss_req_r==1'b0) & (tx_de_req_r==1'b0) ? tbc_line_r[11:0] :
			(tx_ss_req_r==1'b1) ? 12'b0 :
			(tx_ss_req_r==1'b0) & (tbc_line_end_r==1'b1) & (tx_hs_req_r==1'b1) ? 12'b0 :
			(tx_ss_req_r==1'b0) & (tbc_line_end_r==1'b1) & (tx_hs_req_r==1'b0) ? tbc_line_r[11:0] :
			(tx_ss_req_r==1'b0) & (tbc_line_end_r==1'b0) & (tx_de_req_r==1'b1) ? tbc_line_r[11:0]+12'h01 :
			(tx_ss_req_r==1'b0) & (tbc_line_end_r==1'b0) & (tx_de_req_r==1'b0) ? tbc_line_r[11:0] :
			12'h00;

	assign tbc_line_end_w=(tbc_line_r[11:0]==dp_vheight[11:0]) ? 1'b1 : 1'b0;

	assign tbc_data_sel_w[3:0]=
			(tbc_state_r[3:0]==4'b0000) & (tx_de_req_r==1'b1) ? 4'b0001 :
			(tbc_state_r[3:0]==4'b0000) & (tx_de_req_r==1'b0) & (tx_vs_req_r==1'b1) ? 4'b0010 :
			(tbc_state_r[3:0]==4'b0000) & (tx_de_req_r==1'b0) & (tx_vs_req_r==1'b0) &  ((tbc_line_end_r==1'b1) & (tx_hs_req_r==1'b1)) ? 4'b0011 :
			(tbc_state_r[3:0]==4'b0000) & (tx_de_req_r==1'b0) & (tx_vs_req_r==1'b0) & !((tbc_line_end_r==1'b1) & (tx_hs_req_r==1'b1)) ? 4'b0000 :
			(tbc_state_r[3:0]==4'b0001) ? 4'b0010 :
			(tbc_state_r[3:0]==4'b0010) ? 4'b0011 :
			(tbc_state_r[3:0]==4'b0011) ? 4'b0011 :
			(tbc_state_r[3:0]==4'b0100) ? 4'b0010 :
			(tbc_state_r[3:0]==4'b0101) ? 4'b0010 :
			(tbc_state_r[3:0]==4'b0110) ? 4'b0011 :
			(tbc_state_r[3:0]==4'b0111) ? 4'b0011 :
			(tbc_state_r[3]  ==1'b1   ) ? 4'b0001 :
			4'b0000;

	assign tbc_count_w[7:0]=
			(tbc_state_r[3:0]==4'b0000) ? 8'h0 :
			(tbc_state_r[3:0]==4'b0001) ? tbc_count_r[7:0]+8'h01 :
			(tbc_state_r[3:0]==4'b0010) ? 8'h0 :
			(tbc_state_r[3:0]==4'b0011) ? tbc_count_r[7:0]+8'h01 :
			(tbc_state_r[3:0]==4'b0010) ? 8'h0 :
			(tbc_state_r[3:0]==4'b0011) ? 8'h0 :
			(tbc_state_r[3:0]==4'b0110) ? 8'h0 :
			(tbc_state_r[3:0]==4'b0111) ? 8'h0 :
			(tbc_state_r[3]==4'b1) ? tbc_count_r[7:0]+8'h01 :
			8'h0;

	assign tbc_count32_w=
			(tbc_state_r[3:0]==4'b0000) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0001) & (tbc_count_r[3:0]==4'h0f) ? 1'b1 :
			(tbc_state_r[3:0]==4'b0001) & (tbc_count_r[3:0]!=4'h0f) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0010) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0011) & (tbc_count_r[3:0]==4'h0f) ? 1'b1 :
			(tbc_state_r[3:0]==4'b0011) & (tbc_count_r[3:0]!=4'h0f) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0100) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0101) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0110) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0111) ? 1'b0 :
			(tbc_state_r[3]==1'b1) & (tbc_count_r[4:0]==5'h1f) ? 1'b1 :
			(tbc_state_r[3]==1'b1) & (tbc_count_r[4:0]!=5'h1f) ? 1'b0 :
			1'b0;

	assign tbc_rd_count_w[7:0]=
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]!=2'b11) ? tbc_rd_count_r[7:0] :
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]==2'b11) ? tbc_rd_count_r[7:0]+8'h01 :
			(tbc_state_r[3:0]==4'b1001) & (tbc_count_r[1:0]!=2'b11) ? tbc_rd_count_r[7:0] :
			(tbc_state_r[3:0]==4'b1001) & (tbc_count_r[1:0]==2'b11) & (tbc_de_count_r[1:0]==2'b00) ? tbc_rd_count_r[7:0]+8'h01 :
			(tbc_state_r[3:0]==4'b1001) & (tbc_count_r[1:0]==2'b11) & (tbc_de_count_r[1:0]!=2'b00) ? tbc_rd_count_r[7:0] :
			(tbc_state_r[3:0]==4'b1010) ? tbc_rd_count_r[7:0] :
			(tbc_state_r[3:0]==4'b1011) ? tbc_rd_count_r[7:0] :
			(tbc_state_r[3:0]==4'b1100) ? tbc_rd_count_r[7:0] :
			(tbc_state_r[3:0]==4'b1101) ? tbc_rd_count_r[7:0] :
			(tbc_state_r[3:0]==4'b1111) ? tbc_rd_count_r[7:0] :
			8'h0;

	assign tbc_rd_end_w=(tbc_rd_count_r[7:0]==8'ha0) ? 1'b1 : 1'b0;

	assign tbc_blank_end_w=
			(tbc_state_r[3:0]==4'b1000) ? tbc_blank_end_r :
			(tbc_state_r[3:0]==4'b1001) ? tbc_blank_end_r :
			(tbc_state_r[3:0]==4'b1010) ? tbc_blank_end_r :
			(tbc_state_r[3:0]==4'b1011) ? tbc_blank_end_r :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]!=3'b111) ? tbc_blank_end_r :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]==3'b111) ? 1'b1 :
			(tbc_state_r[3:0]==4'b1101) ? 1'b1 :
			(tbc_state_r[3:0]==4'b1110) ? 1'b1 :
			(tbc_state_r[3:0]==4'b1111) ? 1'b1 :
			1'b0;

	assign tbc_de_count_w[1:0]=
			(tbc_state_r[3:0]==4'b1000) ? tbc_de_count_r[1:0] :
			(tbc_state_r[3:0]==4'b1001) ? tbc_de_count_r[1:0] :
			(tbc_state_r[3:0]==4'b1010) ? tbc_de_count_r[1:0] :
			(tbc_state_r[3:0]==4'b1011) ? tbc_de_count_r[1:0] :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]!=3'b111) ? tbc_de_count_r[1:0] :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]==3'b111) & (tbc_de_count_r[1]==1'b1) ? 2'b00 :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]==3'b111) & (tbc_de_count_r[1]==1'b0) ? tbc_de_count_r[1:0]+2'b01 :
			(tbc_state_r[3:0]==4'b1101) ? tbc_de_count_r[1:0] :
			(tbc_state_r[3:0]==4'b1110) ? tbc_de_count_r[1:0] :
			(tbc_state_r[3:0]==4'b1111) ? tbc_de_count_r[1:0] :
			2'b0;

	assign tbc_rd_data00_w[107:0]=
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]!=2'b11) ? tbc_rd_data00_r[107:0] :
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]==2'b11) & (tbc_de_count_r[1:0]==2'b00) ? tbc_rd_data[107:0] :
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]==2'b11) & (tbc_de_count_r[1:0]!=2'b00) ? tbc_rd_data01_r[107:0] :
			(tbc_state_r[3:0]==4'b1001) ? tbc_rd_data00_r[107:0] :
			(tbc_state_r[3:0]==4'b1010) ? tbc_rd_data00_r[107:0] :
			(tbc_state_r[3:0]==4'b1011) ? tbc_rd_data00_r[107:0] :
			(tbc_state_r[3:0]==4'b1100) ? tbc_rd_data00_r[107:0] :
			(tbc_state_r[3:0]==4'b1101) ? tbc_rd_data00_r[107:0] :
			(tbc_state_r[3:0]==4'b1111) ? tbc_rd_data00_r[107:0] :
			108'h0;

	assign tbc_rd_data01_w[107:0]=
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]!=2'b11) ? tbc_rd_data01_r[107:0] :
			(tbc_state_r[3:0]==4'b1000) & (tbc_count_r[1:0]==2'b11) ? tbc_rd_data[107:0] :
			(tbc_state_r[3:0]==4'b1001) & (tbc_count_r[1:0]!=2'b11) ? tbc_rd_data01_r[107:0] :
			(tbc_state_r[3:0]==4'b1001) & (tbc_count_r[1:0]==2'b11) & (tbc_de_count_r[1:0]==2'b00) ? tbc_rd_data[107:0] :
			(tbc_state_r[3:0]==4'b1001) & (tbc_count_r[1:0]==2'b11) & (tbc_de_count_r[1:0]!=2'b00) ? tbc_rd_data01_r[107:0] :
			(tbc_state_r[3:0]==4'b1010) ? tbc_rd_data01_r[107:0] :
			(tbc_state_r[3:0]==4'b1011) ? tbc_rd_data01_r[107:0] :
			(tbc_state_r[3:0]==4'b1100) ? tbc_rd_data01_r[107:0] :
			(tbc_state_r[3:0]==4'b1101) ? tbc_rd_data01_r[107:0] :
			(tbc_state_r[3:0]==4'b1111) ? tbc_rd_data01_r[107:0] :
			108'h0;

	assign tbc_disp00_w[35:0]=
			(tbc_de_count_r[1:0]==2'b00) ? {1'b0,tbc_rd_data01_r[7:0],1'b0,tbc_rd_data00_r[23:16],1'b0,tbc_rd_data00_r[15:8],1'b0,tbc_rd_data00_r[7:0]} :
			(tbc_de_count_r[1:0]==2'b01) ? {1'b0,tbc_rd_data01_r[15:8],1'b0,tbc_rd_data01_r[7:0],1'b0,tbc_rd_data00_r[23:16],1'b0,tbc_rd_data00_r[15:8]} :
			(tbc_de_count_r[1]  ==1'b1 ) ? {1'b0,tbc_rd_data01_r[23:16],1'b0,tbc_rd_data01_r[15:8],1'b0,tbc_rd_data01_r[7:0],1'b0,tbc_rd_data00_r[23:16]} :
			36'b0;
	assign tbc_disp01_w[35:0]=
			(tbc_de_count_r[1:0]==2'b00) ? {1'b0,tbc_rd_data01_r[31:24],1'b0,tbc_rd_data00_r[47:40],1'b0,tbc_rd_data00_r[39:32],1'b0,tbc_rd_data00_r[31:24]} :
			(tbc_de_count_r[1:0]==2'b01) ? {1'b0,tbc_rd_data01_r[39:32],1'b0,tbc_rd_data01_r[31:24],1'b0,tbc_rd_data00_r[47:40],1'b0,tbc_rd_data00_r[39:32]} :
			(tbc_de_count_r[1]  ==1'b1 ) ? {1'b0,tbc_rd_data01_r[47:40],1'b0,tbc_rd_data01_r[39:32],1'b0,tbc_rd_data01_r[31:24],1'b0,tbc_rd_data00_r[47:40]} :
			36'b0;
	assign tbc_disp02_w[35:0]=
			(tbc_de_count_r[1:0]==2'b00) ? {1'b0,tbc_rd_data01_r[55:48],1'b0,tbc_rd_data00_r[71:64],1'b0,tbc_rd_data00_r[63:56],1'b0,tbc_rd_data00_r[55:48]} :
			(tbc_de_count_r[1:0]==2'b01) ? {1'b0,tbc_rd_data01_r[63:56],1'b0,tbc_rd_data01_r[55:48],1'b0,tbc_rd_data00_r[71:64],1'b0,tbc_rd_data00_r[63:56]} :
			(tbc_de_count_r[1]  ==1'b1 ) ? {1'b0,tbc_rd_data01_r[71:64],1'b0,tbc_rd_data01_r[63:56],1'b0,tbc_rd_data01_r[55:48],1'b0,tbc_rd_data00_r[71:64]} :
			36'b0;
	assign tbc_disp03_w[35:0]=
			(tbc_de_count_r[1:0]==2'b00) ? {1'b0,tbc_rd_data01_r[79:72],1'b0,tbc_rd_data00_r[95:88],1'b0,tbc_rd_data00_r[87:80],1'b0,tbc_rd_data00_r[79:72]} :
			(tbc_de_count_r[1:0]==2'b01) ? {1'b0,tbc_rd_data01_r[87:80],1'b0,tbc_rd_data01_r[79:72],1'b0,tbc_rd_data00_r[95:88],1'b0,tbc_rd_data00_r[87:80]} :
			(tbc_de_count_r[1]  ==1'b1 ) ? {1'b0,tbc_rd_data01_r[95:88],1'b0,tbc_rd_data01_r[87:80],1'b0,tbc_rd_data01_r[79:72],1'b0,tbc_rd_data00_r[95:88]} :
			36'b0;

	assign tbc_data_req_w=
			(tbc_state_r[3:0]==4'b0000) & (tx_vs_req_r==1'b1) ? 1'b1 :
			(tbc_state_r[3:0]==4'b0000) & (tx_vs_req_r==1'b0) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0001) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0010) ? 1'b1 :
			(tbc_state_r[3:0]==4'b0011) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0100) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0101) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0110) ? 1'b0 :
			(tbc_state_r[3:0]==4'b0111) ? 1'b0 :
			(tbc_state_r[3:0]==4'b1000) ? 1'b0 :
			(tbc_state_r[3:0]==4'b1001) ? 1'b0 :
			(tbc_state_r[3:0]==4'b1010) ? 1'b0 :
			(tbc_state_r[3:0]==4'b1011) ? 1'b0 :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]!=3'b111) ? 1'b0 :
			(tbc_state_r[3:0]==4'b1100) & (tbc_count_r[2:0]==3'b111) ? 1'b1 :
			(tbc_state_r[3:0]==4'b1101) ? 1'b0 :
			(tbc_state_r[3:0]==4'b1110) ? 1'b0 :
			(tbc_state_r[3:0]==4'b1111) ? 1'b0 :
			1'b0;

	assign tbc_data00_w[287:0]=
			(tbc_data_sel_r[1:0]==2'b10) ? tbc_sync_vs[287:0] :
			(tbc_data_sel_r[1:0]==2'b00) ? tbc_sync_vs[287:0] :
			(tbc_data_sel_r[1:0]==2'b11) ? tbc_sync_ss0[287:0] :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b0) & (tbc_blank_end_r==1'b0) ? {tbc_disp_de[287:45],tbc_disp00_r[35:0],tbc_disp_de[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b0) & (tbc_blank_end_r==1'b1) ? {tbc_disp[287:45],tbc_disp00_r[35:0],tbc_disp[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b1) & (tbc_line_end_r==1'b0) ? {tbc_disp_hs[287:45],tbc_disp00_r[35:0],tbc_disp_hs[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b1) & (tbc_line_end_r==1'b1) ? {tbc_disp_vs[287:45],tbc_disp00_r[35:0],tbc_disp_vs[8:0]} :
			288'b0;

	assign tbc_data01_w[287:0]=
			(tbc_data_sel_r[1:0]==2'b10) ? tbc_sync_vs[287:0] :
			(tbc_data_sel_r[1:0]==2'b00) ? tbc_sync_vs[287:0] :
			(tbc_data_sel_r[1:0]==2'b11) ? tbc_sync_ss1[287:0] :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b0) & (tbc_blank_end_r==1'b0) ? {tbc_disp_de[287:45],tbc_disp01_r[35:0],tbc_disp_de[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b0) & (tbc_blank_end_r==1'b1) ? {tbc_disp[287:45],tbc_disp01_r[35:0],tbc_disp[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b1) & (tbc_line_end_r==1'b0) ? {tbc_disp_hs[287:45],tbc_disp01_r[35:0],tbc_disp_hs[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b1) & (tbc_line_end_r==1'b1) ? {tbc_disp_vs[287:45],tbc_disp01_r[35:0],tbc_disp_vs[8:0]} :
			288'b0;

	assign tbc_data02_w[287:0]=
			(tbc_data_sel_r[1:0]==2'b10) ? tbc_sync_vs[287:0] :
			(tbc_data_sel_r[1:0]==2'b00) ? tbc_sync_vs[287:0] :
			(tbc_data_sel_r[1:0]==2'b11) ? tbc_sync_ss2[287:0] :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b0) & (tbc_blank_end_r==1'b0) ? {tbc_disp_de[287:45],tbc_disp02_r[35:0],tbc_disp_de[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b0) & (tbc_blank_end_r==1'b1) ? {tbc_disp[287:45],tbc_disp02_r[35:0],tbc_disp[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b1) & (tbc_line_end_r==1'b0) ? {tbc_disp_hs[287:45],tbc_disp02_r[35:0],tbc_disp_hs[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b1) & (tbc_line_end_r==1'b1) ? {tbc_disp_vs[287:45],tbc_disp02_r[35:0],tbc_disp_vs[8:0]} :
			288'b0;

	assign tbc_data03_w[287:0]=
			(tbc_data_sel_r[1:0]==2'b10) ? tbc_sync_vs[287:0] :
			(tbc_data_sel_r[1:0]==2'b00) ? tbc_sync_vs[287:0] :
			(tbc_data_sel_r[1:0]==2'b11) ? tbc_sync_ss3[287:0] :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b0) & (tbc_blank_end_r==1'b0) ? {tbc_disp_de[287:45],tbc_disp03_r[35:0],tbc_disp_de[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b0) & (tbc_blank_end_r==1'b1) ? {tbc_disp[287:45],tbc_disp03_r[35:0],tbc_disp[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b1) & (tbc_line_end_r==1'b0) ? {tbc_disp_hs[287:45],tbc_disp03_r[35:0],tbc_disp_hs[8:0]} :
			(tbc_data_sel_r[1:0]==2'b01) & (tbc_rd_end_r==1'b1) & (tbc_line_end_r==1'b1) ? {tbc_disp_vs[287:45],tbc_disp03_r[35:0],tbc_disp_vs[8:0]} :
			288'b0;

	assign tbc_data00_out_w[287:0]=(tbc_data_req_r==1'b1) ? tbc_data00_r[287:0] : {9'b0,tbc_data00_out_r[287:9]};
	assign tbc_data01_out_w[287:0]=(tbc_data_req_r==1'b1) ? tbc_data01_r[287:0] : {9'b0,tbc_data01_out_r[287:9]};
	assign tbc_data02_out_w[287:0]=(tbc_data_req_r==1'b1) ? tbc_data02_r[287:0] : {9'b0,tbc_data02_out_r[287:9]};
	assign tbc_data03_out_w[287:0]=(tbc_data_req_r==1'b1) ? tbc_data03_r[287:0] : {9'b0,tbc_data03_out_r[287:9]};

	// ---- ----

	assign tbc_disp_de[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,					// 13
			9'b0,9'b0,9'b0,					// 10
			9'b0,9'b0,9'b0,9'h0,			// 6
			code_fs[8:0],					// 5
			9'h0ff,9'h0ff,					// 3
			9'h0ff,9'h0ff,					// 1
			code_be[8:0]};					// 0
	assign tbc_disp[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,					// 13
			9'b0,9'b0,9'b0,					// 10
			9'b0,9'b0,9'b0,9'h0,			// 6
			code_fs[8:0],					// 5
			9'h0ff,9'h0ff,					// 3
			9'h0ff,9'h0ff,					// 1
			code_fe[8:0]};					// 0

	assign tbc_disp_hs[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,					// 13
			9'b0,9'b0,9'b0,9'h0,			// 9
			1'b0,dp_maud[7:0],				// 8
			1'b0,dp_mvid[7:0],				// 7
			1'b0,dp_vbid_hs[7:0],			// 6
			code_bs[8:0],					// 5
			9'b0,9'b0,						// 3
			9'h1,9'h1,						// 1
			code_fe[8:0]};					// 0
	assign tbc_disp_vs[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,					// 13
			9'b0,9'b0,9'b0,9'h0,			// 9
			1'b0,dp_maud[7:0],				// 8
			1'b0,dp_mvid[7:0],				// 7
			1'b0,dp_vbid_vs[7:0],			// 6
			code_bs[8:0],					// 5
			9'h0ff,9'h0ff,					// 3
			9'h0ff,9'h0ff,					// 1
			code_fe[8:0]};					// 0

	assign tbc_sync_vs[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,					// 13
			9'b0,9'b0,9'b0,9'h0,			// 9
			1'b0,dp_maud[7:0],				// 8
			1'b0,dp_mvid[7:0],				// 7
			1'b0,dp_vbid_vs[7:0],			// 6
			code_bs[8:0],					// 5
			9'b0,							// 4
			9'b0,							// 3
			9'b0,							// 2
			9'b0,							// 1
			9'b0};							// 0
	assign tbc_sync_ss0[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,9'h0,			// 12
			code_se[8:0],					// 11
			1'b0,dp_hsync[7:0],				// 10
			1'b0,dp_hsync[15:8],			// 9
			1'b0,dp_vtotal[7:0],			// 8
			1'b0,dp_vtotal[15:8],			// 7
			1'b0,dp_htotal[7:0],			// 6
			1'b0,dp_htotal[15:8],			// 5
			1'b0,dp_mvid[7:0],				// 4
			1'b0,dp_mvid[15:8],				// 3
			1'b0,dp_mvid[23:16],			// 2
			code_ss[8:0],					// 1
			code_ss[8:0]};					// 0
	assign tbc_sync_ss1[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,9'h0,			// 12
			code_se[8:0],					// 11
			1'b0,dp_vsync[7:0],				// 10
			1'b0,dp_vsync[15:8],			// 9
			1'b0,dp_vstart[7:0],			// 8
			1'b0,dp_vstart[15:8],			// 7
			1'b0,dp_hstart[7:0],			// 6
			1'b0,dp_hstart[15:8],			// 5
			1'b0,dp_mvid[7:0],				// 4
			1'b0,dp_mvid[15:8],				// 3
			1'b0,dp_mvid[23:16],			// 2
			code_ss[8:0],					// 1
			code_ss[8:0]};					// 0
	assign tbc_sync_ss2[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,9'h0,			// 12
			code_se[8:0],					// 11
			9'b0,							// 10
			9'b0,							// 9
			1'b0,dp_vheight[7:0],			// 8
			1'b0,dp_vheight[15:8],			// 7
			1'b0,dp_hwidth[7:0],			// 6
			1'b0,dp_hwidth[15:8],			// 5
			1'b0,dp_mvid[7:0],				// 4
			1'b0,dp_mvid[15:8],				// 3
			1'b0,dp_mvid[23:16],			// 2
			code_ss[8:0],					// 1
			code_ss[8:0]};					// 0
	assign tbc_sync_ss3[287:0]={
			144'b0,							// 16
			9'b0,9'b0,9'b0,9'h0,			// 12
			code_se[8:0],					// 11
			9'b0,							// 10
			1'b0,dp_misc1[7:0],				// 9
			1'b0,dp_misc0[7:0],				// 8
			1'b0,dp_nvid[7:0],				// 7
			1'b0,dp_nvid[15:8],				// 6
			1'b0,dp_nvid[23:16],			// 5
			1'b0,dp_mvid[7:0],				// 4
			1'b0,dp_mvid[15:8],				// 3
			1'b0,dp_mvid[23:16],			// 2
			code_ss[8:0],					// 1
			code_ss[8:0]};					// 0


endmodule
