//------------------------------------------------------------------------------
//
//  xcm107_lvds.v : displaport test module
//
//  LICENSE : "as-is"
//  TakeshiNagashima(T.NG) caramelgete@gmail.com
//------------------------------------------------------------------------------
//  2009/nov/30 release 0.0  
//  2010/jan/28 release 0.1  display timinig generate test
//  2010/apr/24 release 0.2  target arriagx -> virtex5lxt
//
//------------------------------------------------------------------------------

//`define lane4
//`define lane2
`define lane1

`include "xcm107_lvds_inc.v"

module xcm107_lvds(
	output			LED_X4,		// out
	output			LED_X1,		// out
	output			LED_LINK,	// out

	inout	[3:0]	INTN,		// inout

	input	[1:0]	RX_P,		// out
	input	[1:0]	RX_N,		// out
	output	[1:0]	TX_P,		// out
	output	[1:0]	TX_N,		// out
	inout			TX_SCL,		// inout
	inout			TX_SDA,		// inout
	input			TX_HPD,		// in
	inout			TX_DIR,		// inout
	input			CLK2,		// in
	inout			CLK3,		// in
	inout			CLK4,		// in
	inout			CLK4B,		// in
	input			SYSRST_N	// in
);

	parameter	K237=9'b1_111_10111;
	parameter	K277=9'b1_111_11011;
	parameter	K280=9'b1_000_11100;
	parameter	K281=9'b1_001_11100;
	parameter	K282=9'b1_010_11100;
	parameter	K283=9'b1_011_11100;
	parameter	K284=9'b1_100_11100;
	parameter	K285=9'b1_101_11100;
	parameter	K286=9'b1_110_11100;
	parameter	K287=9'b1_111_11100;
	parameter	K297=9'b1_111_11101;
	parameter	K307=9'b1_111_11110;
	parameter	D102=9'b0_010_01010;
	parameter	D116=9'b0_110_01011;

	localparam	code_bs  =K285;	// defalut symbol blank start
	localparam	code_be  =K277;	// defalut symbol blank end
	localparam	code_ss  =K282;	// defalut symbol secondary start
	localparam	code_se  =K297;	// defalut symbol secondary end
	localparam	code_fs  =K307;	// defalut symbol fill start
	localparam	code_fe  =K237;	// defalut symbol fill end
	localparam	code_sr  =K280;	// defalut symbol scramble reset
	localparam	code_cpbs=K281;	// defalut symbol content protection bs
	localparam	code_cpsr=K283;	// defalut symbol content protection sr

	localparam	code_ebs =K285;	// enhanced symbol blank start
	localparam	code_ebe =K277;	// enhanced symbol blank end
	localparam	code_ess =K282;	// enhanced symbol secondary start
	localparam	code_ese =K297;	// enhanced symbol secondary end
	localparam	code_efs =K307;	// enhanced symbol fill start
	localparam	code_efe =K237;	// enhanced symbol fill end
	localparam	code_esr =K280;	// enhanced symbol scramble reset
	localparam	code_ecp =K281;	// enhanced symbol content protection
	localparam	code_ebf =K283;	// enhanced symbol 

	wire	CLK27;
	wire	GXB162;
	wire	CLK162;
	wire	CLK135;
	wire	CLK270;
	wire	CLK50;
	wire	RST_N;

	// ---- clock gen ----

	wire	CLK3FB;
	wire	CLK4FB;

	wire	locked0;
	wire	locked1;
	wire	locked2;
	wire	locked3;
	wire	pll_locked;

//	assign CLK27=CLK2;
//	assign RST_N=SYSRST_N;

`ifdef simulation

	assign CLK27=CLK2;
	assign CLK162=CLK1;
	assign RST_N=SYSRST_N;

`else

	assign RST_N=locked0;

xil_plladv_27x162 pll_27x162 (
	.CLKIN1_IN(CLK2),
	.RST_IN(!SYSRST_N),
	.CLKFBOUT_OUT(CLK27),
	.CLKOUT0_OUT(CLK162),
	.LOCKED_OUT(locked0)
);

	assign CLK3FB=CLK162;

`endif

//	assign CLK3FB=CLK162;
	assign CLK4FB=CLK162;

	// ---- aux link ----

	wire	aux_data_tx;
	wire	aux_data_rx;
	wire	aux_data_oe;

	assign TX_SDA=1'bz;
	assign TX_SCL=aux_data_tx;
	assign TX_DIR=aux_data_oe;
	assign aux_data_rx=TX_SDA;

	wire	link_done;
	wire	link_err;
	wire	link_main_active;
	wire	[2:0] link_main_mode;
	wire	[7:0] link_unit;

	wire	link_aux_cmd_req;
	wire	[2:0] link_aux_cmd;
	wire	link_aux_cmd_done;
	wire	[3:0] link_aux_stat_tr1;
	wire	[3:0] link_aux_stat_tr2;
	wire	link_aux_stat_irq1;

	wire	link_aux_tx_req;
	wire	link_aux_tx_ack;
	wire	link_aux_tx_done;
	wire	[3:0] link_aux_tx_cmd;
	wire	[19:0] link_aux_tx_addr;
	wire	[7:0] link_aux_tx_len;
	wire	[7:0] link_aux_tx_data;
	wire	link_aux_tx_i2c_ao;
	wire	link_aux_tx_data_req;
	wire	[19:0] link_aux_tx_select;

	wire	[7:0] link_aux_rx_cmd;
	wire	[7:0] link_aux_rx_data;
	wire	link_aux_rx_cmd_ack;
	wire	link_aux_rx_data_ack;
	wire	link_aux_rx_sync;
	wire	link_aux_rx_stop;
	wire	link_aux_rx_done;

	assign LED_X4=(link_aux_stat_tr1[0]==1'b1) ? 1'b0 : 1'b1;
	assign LED_X1=(link_aux_stat_tr2[0]==1'b1) ? 1'b0 : 1'b1;
	assign LED_LINK=(link_done==1'b1) ? 1'b0 : 1'b1;

`ifdef lane1	// linkup x1
dp_main_linkup1 dp_main_linkup(
`else
`ifdef lane2	// linkup x2
dp_main_linkup2 dp_main_linkup(
`else			// linkup x4
dp_main_linkup dp_main_linkup(
`endif
`endif
	.TX_HPD(TX_HPD),								// in    [LINKUP] sink active

	.link_done(link_done),							// out   [LINKUP] linkup done
	.link_err(link_err),							// out   [LINKUP] linkup error
	.link_main_active(link_main_active),			// out   [LINKUP] transmitter active
	.link_main_mode(link_main_mode[2:0]),			// out   [LINKUP] transmit 00=idle/01=normal/10=tr1/11=tr2
	.link_unit(link_unit[7:0]),						// out   [LINKUP] unit size[7:0]

	.link_aux_cmd_req(link_aux_cmd_req),			// out   [LINKUP] linkup command req
	.link_aux_cmd(link_aux_cmd[2:0]),				// out   [LINKUP] linkup command[2:0]
	.link_aux_cmd_done(link_aux_cmd_done),			// out   [LINKUP] linkup command done
	.link_aux_stat_tr1(link_aux_stat_tr1[3:0]),		// out   [LINKUP] linkup tr1 status
	.link_aux_stat_tr2(link_aux_stat_tr2[3:0]),		// out   [LINKUP] linkup tr2 status
	.link_aux_stat_irq1(link_aux_stat_irq1),		// out   [LINKUP] device service irq vector bit1

	.link_aux_tx_req(link_aux_tx_req),				// out   [AUX] send request
	.link_aux_tx_ack(link_aux_tx_ack),				// in    [AUX] send acknowredge
	.link_aux_tx_done(link_aux_tx_done),			// in    [AUX] send done
	.link_aux_tx_cmd(link_aux_tx_cmd[3:0]),			// out   [AUX] send command
	.link_aux_tx_addr(link_aux_tx_addr[19:0]),		// out   [AUX] send addr
	.link_aux_tx_len(link_aux_tx_len[7:0]),			// out   [AUX] send data length
	.link_aux_tx_data(link_aux_tx_data[7:0]),		// out   [AUX] send data
	.link_aux_tx_i2c_ao(link_aux_tx_i2c_ao),		// out   [AUX] send i2c adress-only

	.link_aux_rx_cmd(link_aux_rx_cmd[7:0]),			// in    [AUX] recieve ack
	.link_aux_rx_data(link_aux_rx_data[7:0]),		// in    [AUX] recieve data
	.link_aux_rx_cmd_ack(link_aux_rx_cmd_ack),		// in    [AUX] recieve cmd acknowredge
	.link_aux_rx_data_ack(link_aux_rx_data_ack),	// in    [AUX] recieve data acknowredge
	.link_aux_rx_done(link_aux_rx_done),			// in    [AUX] recieve done

	.clk27(CLK27),									// in    [AUX] clk 27MHz
	.rst_n(RST_N)									// in    [AUX] #reset
);

dp_aux_tx aux_tx(
	.aux_tx(aux_data_tx),							// out   [AUX] dp aux_tx
	.aux_oe(aux_data_oe),							// out   [AUX] dp aux_tx oe
	.aux_req(link_aux_tx_req),						// in    [AUX] send request
	.aux_ack(link_aux_tx_ack),						// out   [AUX] send acknowredge
	.aux_done(link_aux_tx_done),					// out   [AUX] send done
	.aux_cmd(link_aux_tx_cmd[3:0]),					// in    [AUX] send command
	.aux_addr(link_aux_tx_addr[19:0]),				// in    [AUX] send addr
	.aux_len(link_aux_tx_len[7:0]),					// in    [AUX] send data length
	.aux_data(link_aux_tx_data[7:0]),				// in    [AUX] send data
	.aux_i2c_ao(link_aux_tx_i2c_ao),				// in    [AUX] send i2c adress-only
	.aux_data_req(link_aux_tx_data_req),			// out   [AUX] send data request
	.aux_select(link_aux_tx_select[19:0]),			// out   [AUX] data addr
	.clk27(CLK27),									// in    [AUX] clk 27MHz
	.rst_n(RST_N)									// in    [AUX] #reset
);

dp_aux_rx aux_rx(
	.aux_rx(aux_data_rx),							// in    [AUX] dp aux_rx
	.aux_oe(aux_data_oe),							// in    [AUX] dp aux send/#recieve
	.aux_cmd(link_aux_rx_cmd[7:0]),					// out   [AUX] recieve ack
	.aux_data(link_aux_rx_data[7:0]),				// out   [AUX] recieve data
	.aux_cmd_ack(link_aux_rx_cmd_ack),				// out   [AUX] recieve cmd acknowredge
	.aux_data_ack(link_aux_rx_data_ack),			// out   [AUX] recieve data acknowredge
	.aux_sync(link_aux_rx_sync),					// out   [AUX] recieve sync
	.aux_stop(link_aux_rx_stop),					// out   [AUX] recieve stop
	.aux_done(link_aux_rx_done),					// out   [AUX] recieve done
	.clk27(CLK27),									// in    [AUX] clk 27MHz
	.rst_n(RST_N)									// in    [AUX] #reset
);

	wire	[7:0] TX_RED;
	wire	[7:0] TX_GRN;
	wire	[7:0] TX_BLU;
	wire	TX_HS;
	wire	TX_VS;
	wire	TX_DE;
	wire	TX_CLK;

	// ---- main link : tbc ----

	wire	[23:0] dp_mvid;
	wire	[23:0] dp_nvid;
	wire	[7:0] dp_maud;
	wire	[15:0] dp_htotal;
	wire	[15:0] dp_vtotal;
	wire	[15:0] dp_hstart;
	wire	[15:0] dp_vstart;
	wire	[15:0] dp_hwidth;
	wire	[15:0] dp_vheight;
	wire	[15:0] dp_hsync;
	wire	[15:0] dp_vsync;
	wire	[7:0] dp_misc0;
	wire	[7:0] dp_misc1;

	wire	[8:0] tbc_data0;
	wire	[8:0] tbc_data1;
	wire	[8:0] tbc_data2;
	wire	[8:0] tbc_data3;

`ifdef lane1	// linkup x1
dp_main_crtc1 dp_main_crtc(
`else
`ifdef lane2	// linkup x2
dp_main_crtc2 dp_main_crtc(
`else			// linkup x4
dp_main_crtc dp_main_crtc(
`endif
`endif
	.TX_RED(TX_RED[7:0]),				// out   [CRTC] red[7:0]
	.TX_GRN(TX_GRN[7:0]),				// out   [CRTC] grn[7:0]
	.TX_BLU(TX_BLU[7:0]),				// out   [CRTC] blu[7:0]
	.TX_HS(TX_HS),						// out   [CRTC] hsync
	.TX_VS(TX_VS),						// out   [CRTC] vsync
	.TX_DE(TX_DE),						// out   [CRTC] de/#blank
	.TX_CLK(TX_CLK),					// out   [CRTC] pixel clock

	.dp_mvid(dp_mvid[23:0]),			// in    [CRTC] mvid[23:0]
	.dp_nvid(dp_nvid[23:0]),			// in    [CRTC] nvid[23:0]
	.dp_maud(dp_maud[7:0]),				// in    [CRTC] maud[7:0]
	.dp_htotal(dp_htotal[15:0]),		// in    [CRTC] htotal[15:0]
	.dp_vtotal(dp_vtotal[15:0]),		// in    [CRTC] vtotal[15:0]
	.dp_hstart(dp_hstart[15:0]),		// in    [CRTC] hstart[15:0]
	.dp_vstart(dp_vstart[15:0]),		// in    [CRTC] vstart[15:0]
	.dp_hwidth(dp_hwidth[15:0]),		// in    [CRTC] hwidth[15:0]
	.dp_vheight(dp_vheight[15:0]),		// in    [CRTC] vheight[15:0]
	.dp_hsync(dp_hsync[15:0]),			// in    [CRTC] hsync[15:0] ({hs_pol,hs_wid[14:0]})
	.dp_vsync(dp_vsync[15:0]),			// in    [CRTC] vsync[15:0] ({vs_pol,vs_wid[14:0]})
	.dp_misc0(dp_misc0[7:0]),			// in    [CRTC] misc0[7:0]
	.dp_misc1(dp_misc1[7:0]),			// in    [CRTC] misc1[7:0]

	.tbc_data0(tbc_data0[8:0]),			// out   [TBC] data0[8:0]
	.tbc_data1(tbc_data1[8:0]),			// out   [TBC] data1[8:0]
	.tbc_data2(tbc_data2[8:0]),			// out   [TBC] data2[8:0]
	.tbc_data3(tbc_data3[8:0]),			// out   [TBC] data3[8:0]

//	.tbc_data0(),						// out   [TBC] data0[8:0]
//	.tbc_data1(),						// out   [TBC] data1[8:0]
//	.tbc_data2(),						// out   [TBC] data2[8:0]
//	.tbc_data3(),						// out   [TBC] data3[8:0]

	.CLK(GXB162),						// in    [MAIN] system clock
	.SYSRST_N(RST_N)					// in    [MAIN] #reset
);

	// ---- main link : pixel pack ----


	wire	[35:0] pack_data;
	wire	pack_rd;

	// 640x480 25.175MHz

`ifdef sim_fast

	assign dp_mvid[23:0]=24'h001554;			// strm_clk=27,ls_clk=162
	assign dp_nvid[23:0]=24'h008000;			// M:N=STRM_CLK:LS_CLK
	assign dp_maud[7:0]=8'b0;
	assign dp_htotal[15:0]=16'h0320;		// 800 : HOR TOTAL
	assign dp_vtotal[15:0]=16'h0018;		// 525 : VER TOTAL
	assign dp_hstart[15:0]=16'h0090;		// 144 : HOR SYNC + BACK PORCH
	assign dp_vstart[15:0]=16'h0006;		// 35  : VER SYNC + BACK PORCH
	assign dp_hwidth[15:0]=16'h0280;		// 640 : HOR
	assign dp_vheight[15:0]=16'h0010;		// 480 : VER
	assign dp_hsync[15:0]=16'h0060;			// high,96 : HOR SYNC
	assign dp_vsync[15:0]=16'h0002;			// high,2  : VER SYNC
	assign dp_misc0[7:0]=8'b001_0_0_00_1;	// 8bit,-,vesa,rgb,sync
	assign dp_misc1[7:0]=8'b00000_00_0;		// ,,

`else

	assign dp_mvid[23:0]=24'h001554;		// strm_clk=27,ls_clk=162
	assign dp_nvid[23:0]=24'h008000;		// M:N=STRM_CLK:LS_CLK
	assign dp_maud[7:0]=8'b0;
	assign dp_htotal[15:0]=16'h0320;		// 800 : HOR TOTAL
	assign dp_vtotal[15:0]=16'h020d;		// 525 : VER TOTAL
	assign dp_hstart[15:0]=16'h0090;		// 144 : HOR SYNC + BACK PORCH
	assign dp_vstart[15:0]=16'h0023;		// 35  : VER SYNC + BACK PORCH
	assign dp_hwidth[15:0]=16'h0280;		// 640 : HOR
	assign dp_vheight[15:0]=16'h01e0;		// 480 : VER
	assign dp_hsync[15:0]=16'h0060;			// high,96 : HOR SYNC
	assign dp_vsync[15:0]=16'h0002;			// high,2  : VER SYNC
//	assign dp_misc0[7:0]=8'b000_0_0_00_0;	// 6bit,-,vesa,rgb,async
	assign dp_misc0[7:0]=8'b001_0_0_00_0;	// 8bit,-,vesa,rgb,async
	assign dp_misc1[7:0]=8'b00000_00_0;		// ,,
`endif

	// ---- main link : scramble ----

	wire	[1:0] dp_main_mode;
	wire	dp_main_scr;

	wire	[9:0] sc_enc0_8b10b;
	wire	[9:0] sc_enc1_8b10b;
	wire	[9:0] sc_enc2_8b10b;
	wire	[9:0] sc_enc3_8b10b;

	wire	[8:0] sc_raw0;
	wire	[8:0] sc_raw1;
	wire	[8:0] sc_raw2;
	wire	[8:0] sc_raw3;

	wire	[8:0] sc_data0;
	wire	[8:0] sc_data1;
	wire	[8:0] sc_data2;
	wire	[8:0] sc_data3;
	wire	[1:0] sc_disp0;
	wire	[1:0] sc_disp1;
	wire	[1:0] sc_disp2;
	wire	[1:0] sc_disp3;

	assign dp_main_mode[1:0]=link_main_mode[1:0];
	assign dp_main_scr=link_main_mode[2];

dp_main_scramble dp_main_scramble(
	.dp_main_mode(dp_main_mode[1:0]),	// in    [SCR] link 00=idle/01=disp/10=tr1/11=tr2
	.dp_main_scr(dp_main_scr),			// in    [SCR] data scramble on=1/off=0

	.pack_data0(tbc_data0[8:0]),		// in    [PACK] data0
	.pack_data1(tbc_data1[8:0]),		// in    [PACK] data1
	.pack_data2(tbc_data2[8:0]),		// in    [PACK] data2
	.pack_data3(tbc_data3[8:0]),		// in    [PACK] data3

	.sc_enc0(sc_enc0_8b10b[9:0]),		// out   [SCR] encoded data0[8:0]
	.sc_enc1(sc_enc1_8b10b[9:0]),		// out   [SCR] encoded data1[8:0]
	.sc_enc2(sc_enc2_8b10b[9:0]),		// out   [SCR] encoded data2[8:0]
	.sc_enc3(sc_enc3_8b10b[9:0]),		// out   [SCR] encoded data3[8:0]

	.sc_data0(sc_data0[8:0]),			// out   [SCR] transfer data0[8:0]
	.sc_data1(sc_data1[8:0]),			// out   [SCR] transfer data1[8:0]
	.sc_data2(sc_data2[8:0]),			// out   [SCR] transfer data2[8:0]
	.sc_data3(sc_data3[8:0]),			// out   [SCR] transfer data3[8:0]
	.sc_disp0(sc_disp0[1:0]),			// out   [SCR] transfer disp0[1:0]
	.sc_disp1(sc_disp1[1:0]),			// out   [SCR] transfer disp1[1:0]
	.sc_disp2(sc_disp2[1:0]),			// out   [SCR] transfer disp2[1:0]
	.sc_disp3(sc_disp3[1:0]),			// out   [SCR] transfer disp3[1:0]

	.sc_raw0(sc_raw0[8:0]),				// out   [SCR] raw data0[8:0]
	.sc_raw1(sc_raw1[8:0]),				// out   [SCR] raw data1[8:0]
	.sc_raw2(sc_raw2[8:0]),				// out   [SCR] raw data2[8:0]
	.sc_raw3(sc_raw3[8:0]),				// out   [SCR] raw data3[8:0]

	.CLK(GXB162),						// in    [MAIN] system clock
	.SYSRST_N(RST_N)					// in    [MAIN] #reset
);


`ifdef sim_no_phy

	assign GXB162=CLK3FB;

	assign TX_P[0]=1'b0;
	assign TX_P[1]=1'b0;
	assign TX_P[2]=1'b0;
	assign TX_P[3]=1'b0;

`else

	wire	cal_blk_clk;
	wire	gxb_enable;
	wire	gxb_powerdown;
	wire	pll_inclk;
	wire	[3:0] tx_coreclk;
	wire	[3:0] tx_ctrlenable;
	wire	[31:0] tx_datain;
	wire	[9:0] tx_encin;
	wire	[3:0] tx_digitalreset;
	wire	[3:0] tx_dispval;
	wire	[3:0] tx_forcedisp;
//	wire	pll_locked;
	wire	[3:0] tx_clkout;
	wire	[3:0] tx_dataout;

	assign cal_blk_clk=CLK50;
	assign gxb_enable=1'b1;
	assign gxb_powerdown=1'b0;
	assign pll_inclk=CLK3FB;
	assign tx_coreclk[3:0]=tx_clkout[3:0];
	assign tx_digitalreset[0]=1'b0;

	assign tx_encin[9:0]=sc_enc0_8b10b[9:0];

	assign tx_ctrlenable[0]=sc_data0[8];
	assign tx_ctrlenable[1]=sc_data1[8];
	assign tx_ctrlenable[2]=sc_data2[8];
	assign tx_ctrlenable[3]=sc_data3[8];
	assign tx_datain[7:0]=sc_data0[7:0];
	assign tx_datain[15:8]=sc_data1[7:0];
	assign tx_datain[23:16]=sc_data2[7:0];
	assign tx_datain[31:24]=sc_data3[7:0];
	assign tx_dispval[0]=sc_disp0[0];
	assign tx_dispval[1]=sc_disp1[0];
	assign tx_dispval[2]=sc_disp2[0];
	assign tx_dispval[3]=sc_disp3[0];
	assign tx_forcedisp[0]=sc_disp0[1];
	assign tx_forcedisp[1]=sc_disp1[1];
	assign tx_forcedisp[2]=sc_disp2[1];
	assign tx_forcedisp[3]=sc_disp3[1];

	assign GXB162=tx_clkout[0];


XIL_ROCKETIO_WRAPPER # (
	.WRAPPER_SIM_GTPRESET_SPEEDUP   (0),      // Set this to 1 for simulation
	.WRAPPER_SIM_PLL_PERDIV2        (9'h134)
) rocketio_wrapper_i (
	//_____________________________________________________________________
	//_____________________________________________________________________
	//TILE0  (X0Y1)
	//---------------------- Loopback and Powerdown Ports ----------------------
	.TILE0_LOOPBACK0_IN             (1'b0),
	.TILE0_LOOPBACK1_IN             (1'b0),
	//--------------------- Receive Ports - 8b10b Decoder ----------------------
	.TILE0_RXCHARISK0_OUT           (),
	.TILE0_RXCHARISK1_OUT           (),
	.TILE0_RXDISPERR0_OUT           (),
	.TILE0_RXDISPERR1_OUT           (),
	.TILE0_RXNOTINTABLE0_OUT        (),
	.TILE0_RXNOTINTABLE1_OUT        (),
	.TILE0_RXRUNDISP0_OUT           (),
	.TILE0_RXRUNDISP1_OUT           (),
	//------------- Receive Ports - Comma Detection and Alignment --------------
	.TILE0_RXENMCOMMAALIGN0_IN      (1'b1),
	.TILE0_RXENMCOMMAALIGN1_IN      (1'b1),
	.TILE0_RXENPCOMMAALIGN0_IN      (1'b1),
	.TILE0_RXENPCOMMAALIGN1_IN      (1'b1),
	//----------------- Receive Ports - RX Data Path interface -----------------
	.TILE0_RXDATA0_OUT              (),
	.TILE0_RXDATA1_OUT              (),
	.TILE0_RXRECCLK0_OUT            (),
	.TILE0_RXRECCLK1_OUT            (),
	.TILE0_RXRESET0_IN              (!RST_N),
	.TILE0_RXRESET1_IN              (!RST_N),
	.TILE0_RXUSRCLK0_IN             (pll_inclk),
	.TILE0_RXUSRCLK1_IN             (pll_inclk),
	.TILE0_RXUSRCLK20_IN            (pll_inclk),
	.TILE0_RXUSRCLK21_IN            (pll_inclk),
	//----- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
	.TILE0_RXEQMIX0_IN              (1'b0),
	.TILE0_RXEQMIX1_IN              (1'b0),
	.TILE0_RXEQPOLE0_IN             (1'b0),
	.TILE0_RXEQPOLE1_IN             (1'b0),
	.TILE0_RXN0_IN                  (RX_N[0]),
	.TILE0_RXN1_IN                  (RX_N[1]),
	.TILE0_RXP0_IN                  (RX_P[0]),
	.TILE0_RXP1_IN                  (RX_P[1]),
	//------------- Receive Ports - RX Loss-of-sync State Machine --------------
	.TILE0_RXLOSSOFSYNC0_OUT        (),
	.TILE0_RXLOSSOFSYNC1_OUT        (),
	//------------------- Shared Ports - Tile and PLL Ports --------------------
	.TILE0_CLKIN_IN                 (pll_inclk),
	.TILE0_GTPRESET_IN              (!RST_N),
	.TILE0_PLLLKDET_OUT             (pll_locked),
	.TILE0_REFCLKOUT_OUT            (),
	.TILE0_RESETDONE0_OUT           (),
	.TILE0_RESETDONE1_OUT           (),
	//-------------- Transmit Ports - 8b10b Encoder Control Ports --------------
	.TILE0_TXCHARDISPMODE0_IN       (1'b0),
	.TILE0_TXCHARDISPMODE1_IN       (tx_forcedisp[0]),
	.TILE0_TXCHARDISPVAL0_IN        (1'b0),
	.TILE0_TXCHARDISPVAL1_IN        (tx_dispval[0]),
	.TILE0_TXCHARISK0_IN            (K285[8]),
	.TILE0_TXCHARISK1_IN            (tx_ctrlenable[0]),
	//---------------- Transmit Ports - TX Data Path interface -----------------
	.TILE0_TXDATA0_IN               (K285[7:0]),
	.TILE0_TXDATA1_IN               (tx_datain[7:0]),
	.TILE0_TXOUTCLK0_OUT            (),
	.TILE0_TXOUTCLK1_OUT            (tx_clkout[0]),
	.TILE0_TXUSRCLK0_IN             (pll_inclk),
	.TILE0_TXUSRCLK1_IN             (pll_inclk),
	.TILE0_TXUSRCLK20_IN            (pll_inclk),
	.TILE0_TXUSRCLK21_IN            (pll_inclk),
	//------------- Transmit Ports - TX Driver and OOB signalling --------------
	.TILE0_TXDIFFCTRL0_IN           (3'b010),
	.TILE0_TXDIFFCTRL1_IN           (3'b010),	// 800mv
	.TILE0_TXN0_OUT                 (TX_N[0]),
	.TILE0_TXN1_OUT                 (TX_N[1]),
	.TILE0_TXP0_OUT                 (TX_P[0]),
	.TILE0_TXP1_OUT                 (TX_P[1]),
	.TILE0_TXPREEMPHASIS0_IN        (4'b0011),
	.TILE0_TXPREEMPHASIS1_IN        (4'b0011)	// 25%
);

`endif

	assign INTN[0]=TX_DIR;
	assign INTN[1]=TX_SCL;
	assign INTN[2]=TX_SDA;
	assign INTN[3]=TX_HPD;

/*
	reg		[31:0] count_r;
	wire	[31:0] count_w;

	always @(posedge CLK27 or negedge SYSRST_N)
	begin
		if (SYSRST_N==1'b0)
			begin
				count_r[31:0] <= 32'b0;
			end
		else
			begin
				count_r[31:0] <= count_w[31:0];
			end
	end

	assign count_w[31:0]=count_r[31:0]+32'h01;
*/

endmodule
